package com.hisschemoller.audio.controller 
{
	import com.hisschemoller.audio.model.AudioEngineProxy;
	import com.hisschemoller.audio.model.SamplerInstrumentProxy;
	import com.hisschemoller.audio.model.enum.AudioProcessorType;
	import com.hisschemoller.audio.model.processor.IAudioInstrument;
	import com.hisschemoller.ui.model.vo.AlertVO;
	import com.hisschemoller.ui.notification.UINotifications;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.command.SimpleCommand;
	import org.puremvc.as3.multicore.patterns.proxy.Proxy;

	/**
	 * @author Wouter Hisschemoller
	 */
	public class AddNewInstrumentCommand extends SimpleCommand 
	{
		override public function execute (notification : INotification) : void
		{
			var instrumentXML : XML = notification.getBody( ) as XML;
			var audioEngineProxy : AudioEngineProxy = facade.retrieveProxy( AudioEngineProxy.NAME ) as AudioEngineProxy;
			
			/** Create the instrument. */
			switch( String( instrumentXML.@type ) )
			{
				case AudioProcessorType.PITCHED_SAMPLE_PROCESSOR:
					var instrument : SamplerInstrumentProxy = new SamplerInstrumentProxy( (instrumentXML.@id) );
					break;
					
				default:
					facade.sendNotification( UINotifications.ALERT, new AlertVO( "Wrong type", "An instrument defined in the settings file is of an unsupported type: " + instrumentXML.@type + ". Please correct the settings file." ) );
					log("e Unknown instrument type: " + instrumentXML.@type);
					return;
			}
			
			/** Loop through the instrument programs XMLList. */
			var programXMLList : XMLList = instrumentXML..program;
			var n : uint = programXMLList.length();
			while(--n > -1)
			{
				var programXML : XML = programXMLList[ n ];
				instrument.addProgram( programXML );
			}
			
			/** Register the instrument proxy. */
			facade.registerProxy( instrument as Proxy );
			
			/** Add the instrument to the audio engine. */
			audioEngineProxy.addInstrument( instrument as IAudioInstrument );
			
			/** Send program change to instrument. */
			// var message : ChannelMessage = new ChannelMessage(MessageStatus.PROGRAM_CHANGE, 0, int( instrumentXML.@selectedprogram ), 0);
			// var audioEventVO : AudioEventVO = new AudioEventVO(message, instrument.id, 0);
			// sendNotification(AudioNotifications.MESSAGE_TO_INSTRUMENT, audioEventVO);
		}
	}
}
