package com.hisschemoller.audio.controller
{
	import com.hisschemoller.audio.model.AudioEngineProxy;
	import com.hisschemoller.audio.model.processor.IAudioInstrument;
	import com.hisschemoller.audio.model.vo.AudioEventVO;
	import com.newgonzo.midi.messages.ChannelMessage;
	import com.newgonzo.midi.messages.MessageStatus;
	import com.newgonzo.midi.messages.VoiceMessage;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.command.SimpleCommand;
	/**
	 * @author wouter.hisschemoller
	 * 
	 * Sends NOTE_ON, NOTE_OFF and PROGRAM_CHANGE messages to an instrument in the audio engine.
	 * These messages are sent by the sequencer while it's playing.
	 */
	public class MessageToInstrumentCommand extends SimpleCommand
	{
		override public function execute (note : INotification) : void 
		{
			var eventVO : AudioEventVO = AudioEventVO( note.getBody( ) );
			var audioEngineProxy : AudioEngineProxy = AudioEngineProxy( facade.retrieveProxy( AudioEngineProxy.NAME ) );
			var instrument : IAudioInstrument = audioEngineProxy.getInstrumentByID( eventVO.instrumentID );
			
			switch( eventVO.message.status )
			{
				case MessageStatus.NOTE_ON:
					instrument.noteOn( VoiceMessage( eventVO.message ), eventVO.deltaTime );
					break;
				
				case MessageStatus.NOTE_OFF:
					instrument.noteOff( VoiceMessage( eventVO.message ), eventVO.deltaTime );
					break;
					
				case MessageStatus.PROGRAM_CHANGE:
					instrument.programChange( ChannelMessage( eventVO.message ), eventVO.deltaTime );
					break;
					
				default:
					log( "e Message status not recognized: " + eventVO.message.status );
					break;
			}
		}
	}
}
