package com.hisschemoller.audio.model 
{
	import com.hisschemoller.audio.model.processor.IAudioInstrument;

	import org.puremvc.as3.multicore.interfaces.IProxy;
	import org.puremvc.as3.multicore.patterns.proxy.Proxy;

	import flash.utils.getQualifiedClassName;

	/**
	 * @author Wouter Hisschemöller
	 * Jul 17, 2009
	 * 
	 * Manages instruments.
	 * Instruments can be added to and removed from the engine.
	 * Once added an instrument's process() function is called each time a buffer
	 * of samples is requested by the system so the instrument can add it's audio
	 * data to the buffer.
	 */
	public class AudioEngineProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = getQualifiedClassName( AudioEngineProxy );
		private var _instruments : Vector.<IAudioInstrument> = new Vector.<IAudioInstrument>( );
		private var _selectedInstrumentIndex : int = -1;
		private var _selectedInstrument : IAudioInstrument;

		public function AudioEngineProxy ()
		{
			super( NAME );
		}

		/**
		 * Called by audio driver. Let all instruments contribute their audio data to the buffer.
		 * @param buffer : stereo buffer of AudioDriver.BUFFER_SIZE number of samples
		 * @param bufferStartIndex : From where in the buffer to start to write.
		 * @param bufferLength : Number of samples to write.
		 */
		public function process ( buffer : Vector.<Vector.<Number>>, bufferStartIndex : uint, bufferLength : uint ) : void
		{
			var n : int = _instruments.length;
			while(--n > -1)
			{
				_instruments[n].process( buffer, bufferStartIndex, bufferLength );
			}
		}

		/**
		 * Add an instrument to the engine so it can play sounds.
		 */
		public function addInstrument (instrument : IAudioInstrument) : void
		{
			_instruments.push( instrument );
			
			selectInstrumentByIndex( _instruments.length - 1 );
		}

		/**
		 * Remove an instrument from the engine.
		 */
		public function removeInstrumentByID ( id : String ) : void
		{
			/** Find and remove the instrument from the engine. */
			var n : int = _instruments.length;
			while(--n > -1)
			{
				if(_instruments[n].id == id)
				{
					_instruments.splice( n, 1 );
					break;
				}
			}
			
			/** If the selected instrument is the one that was removed.  */
			if(_selectedInstrument == null)
			{
				_selectedInstrumentIndex = -1;
			}
		}

		/**
		 * Select instrument bsed on it's ID.
		 */
		public function selectInstrumentByID (id : String) : int
		{
			/** If the instrument with this ID is already selected. */
			if(_selectedInstrument && (id == _selectedInstrument.id)) return -1;
			
			/** Find the instrument. */
			var n : int = _instruments.length;
			while(--n > -1)
			{
				if(_instruments[n].id == id)
				{
					var index : int = selectInstrumentByIndex( n );
					return index;
				}
			}
			
			return -1;
		}

		/**
		 * Return instrument based on it's ID.
		 */
		public function getInstrumentByID (instrumentID : String) : IAudioInstrument
		{
			var n : int = _instruments.length;
			while(--n > -1)
			{
				if(_instruments[n].id == instrumentID)
				{
					return _instruments[n];
				}
			}
			
			return null;
		}

		/**
		 * Set selected instrument based on it's position in the _instruments list.
		 */
		private function selectInstrumentByIndex (index : uint) : int
		{
			if(index >= _instruments.length) return -1;
			
			_selectedInstrumentIndex = index;
			_selectedInstrument = _instruments[_selectedInstrumentIndex];
			
			return index;
		}

		/**
		 * Return the selected instrument.
		 */
		public function get selectedInstrument () : IAudioInstrument
		{
			return _selectedInstrument;
		}
	}
}
