package com.hisschemoller.sequencer.controller.euclid 
{
	import com.hisschemoller.sequencer.model.SequencerProxy;
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.model.vo.SequenceVO;
	import com.hisschemoller.sequencer.model.vo.TempoVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.utils.GUID;
	import com.newgonzo.midi.messages.MessageStatus;
	import com.newgonzo.midi.messages.VoiceMessage;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.command.SimpleCommand;

	import flash.geom.Point;

	/**
	 * @author wouterhisschemoller
	 */
	public class CreateEuclidPatternCommand extends SimpleCommand 
	{
		/**
		 * Create a new Euclid Pattern.
		 * For now, when a new pattern is created it will play the first track
		 * by default. A different track may be chosen in the track settings.
		 */
		override public function execute ( note : INotification ) : void
		{
			var point : Point = note.getBody( ) as Point;
			
			var sequencerProxy : SequencerProxy = facade.retrieveProxy( SequencerProxy.NAME ) as SequencerProxy;
			var tempoVO : TempoVO = sequencerProxy.tempoVO;
			var sequenceVO : SequenceVO = sequencerProxy.selectedSequence;
			
			/** Get ID of first track as default. */
			if( sequencerProxy.tracks.length > 0 )
			{
				var trackID : String = sequencerProxy.tracks[0].id;
			}
			
			/** Create a default pattern. */
			var patternVO : EuclidPatternVO = new EuclidPatternVO( );
			patternVO.shape.x = point.x;
			patternVO.shape.y = point.y;
			patternVO.steps = 16;
			patternVO.fills = 4;
			patternVO.id = GUID.create();
			patternVO.trackID = trackID;
			patternVO.quantization = tempoVO.partsPerQuarterNote / 4;
			patternVO.length = patternVO.steps * patternVO.quantization;
			patternVO.currentEventIndex = 0;
			
			/** Add four events to the pattern. */
			for( var i : uint = 0; i < patternVO.fills ; i++)
			{
				var seqEventVO : SeqEventVO = new SeqEventVO( );
				seqEventVO.time = i * 4 * patternVO.quantization;
				seqEventVO.trackID = trackID;
				seqEventVO.message = new VoiceMessage(MessageStatus.NOTE_ON, 0);
				
				patternVO.events.push( seqEventVO );
			}
			
			/** Add pattern to sequence. */
			sequenceVO.patterns.push( patternVO );
			
			/** Send notification for editor to draw pattern. */
			sendNotification( SeqNotifications.SEQ_EUCLID_PATTERN_CREATED, patternVO );
			sendNotification( SeqNotifications.SEQ_SELECT_PATTERN, patternVO );
		}
	}
}
