package com.hisschemoller.sequencer.controller.euclid 
{
	import com.hisschemoller.sequencer.model.SequencerProxy;
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.model.vo.SequenceVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.utils.EuclidGenerator;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.command.SimpleCommand;
	/**
	 * @author wouterhisschemoller
	 */
	public class UpdateEuclidPatternCommand extends SimpleCommand 
	{
		/**
		 * Update pattern with changed data.
		 */
		override public function execute ( note : INotification ) : void
		{
			var settings : EuclidPatternVO = note.getBody( ) as EuclidPatternVO;
			var patternChanged : Boolean = false;
			var positionChanged : Boolean = false;
			
			var sequencerProxy : SequencerProxy = facade.retrieveProxy( SequencerProxy.NAME ) as SequencerProxy;
			var sequenceVO : SequenceVO = sequencerProxy.selectedSequence;
			
			/** Search for the pattern in the selected sequence. */
			var n : int = sequenceVO.patterns.length;
			while( --n > -1 )
			{
				if( settings.id == sequenceVO.patterns[n].id )
				{
					var patternVO : EuclidPatternVO = sequenceVO.patterns[n] as EuclidPatternVO;
					break;
				}
			}
			
			/** If the pattern wasn't found quit. */
			if( !patternVO )
			{
				return;
			}
			
			/** Regenerate the pattern. */
			if( settings.steps != 0 && settings.fills != 0 )
			{
				/** Check if the values are actually changed. */
				if( ( settings.steps != patternVO.steps ) || ( settings.fills != patternVO.events.length ) )
				{
					patternChanged = true;
					
					/** Adjust length (measured in PPQN). */
					patternVO.length = settings.steps * patternVO.quantization;
					
					/** If the index exceeds the new number of steps reset it to zero. */
					if( patternVO.currentEventIndex >= settings.steps )
					{
						patternVO.currentEventIndex = 0;
					}
					
					/** Generate the Euclid / Bjorklund pattern. */
					var euclid : EuclidGenerator = new EuclidGenerator( settings.steps, settings.fills );
					
					/** Take first event in patternVO as 'blueprint'. */
					var sourceEventVO : SeqEventVO = patternVO.events[ 0 ].clone( );
					
					/** Clear all events. */
					patternVO.events = new Vector.<SeqEventVO>( );
					
					/** Loop through the new euclid pattern */
					n = euclid.pattern.length;
					for( var i : uint = 0; i < n ; i++ )
					{
						if( euclid.pattern[ i ] == true )
						{
							var seqEventVO : SeqEventVO = sourceEventVO.clone( );
							seqEventVO.time = i * patternVO.quantization;
							patternVO.events.push( seqEventVO );
						}
					}
				}
			}
			
			/** Update the pattern with the values from the settings pattern. */
			if(settings.steps != 0)
			{
				patternVO.steps = settings.steps;
				patternChanged = true;
			}
			
			if(settings.fills != 0)
			{
				patternVO.fills = settings.fills;
				patternChanged = true;
			}
			
			if(settings.shape.x)
			{
				patternVO.shape.x = settings.shape.x;
				positionChanged = true;
			}
			
			if(settings.shape.y)
			{
				patternVO.shape.y = settings.shape.y;
				positionChanged = true;
			}
			
			if( patternChanged )
			{
				sendNotification( SeqNotifications.SEQ_EUCLID_PATTERN_UPDATED, patternVO );
			}
		}
	}
}
