package com.hisschemoller.sequencer.view
{
	import com.hisschemoller.audio.model.AudioOutputProxy;
	import com.hisschemoller.audio.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.audio.notification.AudioNotifications;
	import com.hisschemoller.sequencer.model.enum.PlaybackStatus;
	import com.hisschemoller.sequencer.model.vo.TempoVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.sequencer.view.components.Controls;
	import com.hisschemoller.sequencer.view.enum.UIEventType;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.mediator.Mediator;

	import flash.events.Event;
	import flash.utils.getQualifiedClassName;

	/**
	 * @author wouter.hisschemoller
	 * Oct 13, 2010
	 */
	public class ControlsMediator extends Mediator
	{
		public static const NAME : String = getQualifiedClassName( ControlsMediator );

		public function ControlsMediator ( viewComponent : Controls )
		{
			super( NAME, viewComponent );
		}

		/**
		 * 
		 */
		override public function listNotificationInterests () : Array
		{
			return [ AudioNotifications.ENTER_FRAME,
					 SeqNotifications.SEQ_PLAYBACK_CHANGED, 
					 SeqNotifications.SEQ_TEMPO_CHANGED, 
					 AudioNotifications.AUDIO_OUTPUT_GAIN_CHANGED ];
		}

		/**
		 * 
		 */
		override public function handleNotification ( notification : INotification ) : void
		{
			switch(notification.getName( ))
			{
				case AudioNotifications.ENTER_FRAME:
					view.updateMeters( notification.getBody( ) as AudioStreamInfoVO );
					break;
				case SeqNotifications.SEQ_PLAYBACK_CHANGED:
					view.updatePlaybackState( notification.getBody( ) as String );
					break;
				case SeqNotifications.SEQ_TEMPO_CHANGED:
					view.updateTempo( (notification.getBody( ) as TempoVO).beatsPerMinute.value );
					break;
				case AudioNotifications.AUDIO_OUTPUT_GAIN_CHANGED:
					view.updateVolumeNormalized( notification.getBody( ) as Number );
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister () : void
		{
			view.addEventListener( UIEventType.PLAYBACK_START, UIEventHandler );
			view.addEventListener( UIEventType.PLAYBACK_STOP, UIEventHandler );
			view.addEventListener( UIEventType.MASTER_VOLUME_CHANGE, UIEventHandler );
			view.addEventListener( UIEventType.TEMPO_CHANGE, UIEventHandler );
		}

		/**
		 * 
		 */
		override public function onRemove () : void
		{
			view.removeEventListener( UIEventType.PLAYBACK_START, UIEventHandler );
			view.removeEventListener( UIEventType.PLAYBACK_STOP, UIEventHandler );
			view.removeEventListener( UIEventType.MASTER_VOLUME_CHANGE, UIEventHandler );
			view.removeEventListener( UIEventType.TEMPO_CHANGE, UIEventHandler );
		}

		/**
		 * 
		 */
		private function UIEventHandler ( event : Event ) : void
		{
			switch(event.type)
			{
				case UIEventType.PLAYBACK_START:
					sendNotification( SeqNotifications.SEQ_CHANGE_PLAYBACK, PlaybackStatus.PLAYING );
					break;
				case UIEventType.PLAYBACK_STOP:
					sendNotification( SeqNotifications.SEQ_CHANGE_PLAYBACK, PlaybackStatus.STOPPED );
					break;
				case UIEventType.MASTER_VOLUME_CHANGE:
					(facade.retrieveProxy( AudioOutputProxy.NAME ) as AudioOutputProxy).decibelNormalized = view.getVolumeKnobValueNormalized( );
					break;
				case UIEventType.TEMPO_CHANGE:
					sendNotification( SeqNotifications.SEQ_CHANGE_TEMPO, view.getTempoKnobValue( ) );
					break;
			}
		}

		/**
		 * 
		 */
		private function get view () : Controls
		{
			return getViewComponent( ) as Controls;
		}
	}
}
