package com.hisschemoller.sequencer.view 
{
	import com.hisschemoller.audio.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.audio.notification.AudioNotifications;
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternViewVO;
	import com.hisschemoller.sequencer.model.vo.TempoVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.sequencer.view.components.euclidpattern.EuclidEditor;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.events.EuclidPatternEvent;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.mediator.Mediator;

	import flash.events.Event;
	import flash.geom.Point;
	import flash.utils.getQualifiedClassName;
	/**
	 * @author wouterhisschemoller
	 */
	public class EuclidEditorMediator extends Mediator 
	{
		public static const NAME : String = getQualifiedClassName( EuclidEditorMediator );

		public function EuclidEditorMediator (viewComponent : EuclidEditor)
		{
			super( NAME, viewComponent );
		}

		/**
		 * 
		 */
		override public function listNotificationInterests () : Array
		{
			return [ AudioNotifications.ENTER_FRAME, 
					 SeqNotifications.SEQ_UPDATE_PATTERN_VIEW, 
					 SeqNotifications.SEQ_SELECT_PATTERN, 
					 SeqNotifications.SEQ_EUCLID_PATTERN_CREATED, 
					 SeqNotifications.SEQ_EUCLID_PATTERN_UPDATED,
					 SeqNotifications.SEQ_TEMPO_CHANGED ];
		}

		/**
		 * 
		 */
		override public function handleNotification ( notification : INotification ) : void
		{
			switch(notification.getName( ))
			{
				case AudioNotifications.ENTER_FRAME:
					view.updateTime( (notification.getBody( ) as AudioStreamInfoVO).latencyInMS );
					break;
					
				case SeqNotifications.SEQ_UPDATE_PATTERN_VIEW:
					view.updateViewData( notification.getBody( ) as PatternViewVO );
					break;
					
				case SeqNotifications.SEQ_SELECT_PATTERN:
					view.selectPattern( notification.getBody( ) as PatternVO );
					break;
					
				case SeqNotifications.SEQ_EUCLID_PATTERN_CREATED:
					view.addPattern( notification.getBody( ) as EuclidPatternVO );
					break;
					
				case SeqNotifications.SEQ_EUCLID_PATTERN_UPDATED:
					view.updatePattern( notification.getBody( ) as EuclidPatternVO );
					break;
					
				case SeqNotifications.SEQ_TEMPO_CHANGED:
					view.updateTempo( notification.getBody( ) as TempoVO );
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister () : void
		{
			view.addEventListener( UIEventType.CREATE_PATTERN, viewEventHandler );
			view.addEventListener( EuclidPatternEvent.EDIT_PATTERN_SETTINGS, viewEventHandler );
		}

		/**
		 * 
		 */
		override public function onRemove () : void
		{
			view.removeEventListener( UIEventType.CREATE_PATTERN, viewEventHandler );
			view.removeEventListener( EuclidPatternEvent.EDIT_PATTERN_SETTINGS, viewEventHandler );
		}

		/**
		 * 
		 */
		private function viewEventHandler ( event : Event ) : void
		{
			switch( event.type )
			{
				case UIEventType.CREATE_PATTERN:
					sendNotification( SeqNotifications.SEQ_CREATE_EUCLID_PATTERN, new Point( view.mouseX, view.mouseY ) );
					break;
					
				case EuclidPatternEvent.EDIT_PATTERN_SETTINGS:
					sendNotification( SeqNotifications.SEQ_EDIT_PATTERN_SETTINGS, ( event as EuclidPatternEvent ).patternID );
					break;
			}
		}

		/** 
		 * 
		 */
		private function get view () : EuclidEditor
		{
			return viewComponent as EuclidEditor;
		}
	}
}
