package com.hisschemoller.sequencer.view 
{
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.TrackVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.sequencer.view.components.settings.EuclidPatternSettings;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.ui.model.vo.AlertVO;
	import com.hisschemoller.ui.notification.UINotifications;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.mediator.Mediator;

	import flash.events.Event;
	import flash.utils.getQualifiedClassName;
	/**
	 * @author wouterhisschemoller
	 */
	public class EuclidPatternSettingsMediator extends Mediator 
	{
		public static const NAME : String = getQualifiedClassName( EuclidPatternSettingsMediator );

		public function EuclidPatternSettingsMediator ( viewComponent : EuclidPatternSettings )
		{
			super( NAME, viewComponent );
		}

		/**
		 * 
		 */
		override public function listNotificationInterests () : Array
		{
			return [ SeqNotifications.SEQ_SELECT_PATTERN, 
					 SeqNotifications.SEQ_TRACK_ADDED ];
		}

		/**
		 * 
		 */
		override public function handleNotification ( notification : INotification ) : void
		{
			switch(notification.getName( ))
			{
				case SeqNotifications.SEQ_SELECT_PATTERN:
					view.updatePattern( notification.getBody( ) as EuclidPatternVO );
					break;
					
				case SeqNotifications.SEQ_TRACK_ADDED:
					view.addTrackToComboBox( notification.getBody( ) as TrackVO  );
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister () : void
		{
			view.addEventListener( UIEventType.TRACK_DELETE, UIEventHandler );
			view.addEventListener( UIEventType.PATTERN_CHANGE, UIEventHandler );
			view.addEventListener( UIEventType.TRACK_CHANGE, UIEventHandler );
		}

		/**
		 * 
		 */
		override public function onRemove () : void
		{
			view.removeEventListener( UIEventType.TRACK_DELETE, UIEventHandler );
			view.removeEventListener( UIEventType.PATTERN_CHANGE, UIEventHandler );
			view.removeEventListener( UIEventType.TRACK_CHANGE, UIEventHandler );
		}

		/**
		 * 
		 */
		private function UIEventHandler ( event : Event ) : void
		{
			switch(event.type)
			{
				case UIEventType.TRACK_DELETE:
					sendNotification( UINotifications.ALERT, new AlertVO( "Delete track", "Are you sure you want to delete this track? This can't be undone." ) );
					break;
					
				case UIEventType.PATTERN_CHANGE:
					sendNotification( SeqNotifications.SEQ_UPDATE_EUCLID_PATTERN, view.getSettings() );
					break;
					
				case UIEventType.TRACK_CHANGE:
					sendNotification( SeqNotifications.SEQ_UPDATE_PATTERN, view.getTrackSettings() );
					break;
			}
		}

		/**
		 * 
		 */
		private function get view () : EuclidPatternSettings
		{
			return getViewComponent( ) as EuclidPatternSettings;
		}
	}
}
