
package com.hisschemoller.sequencer.view.components
{
	import com.bit101.components.Knob;
	import com.bit101.components.Label;
	import com.bit101.components.Meter;
	import com.bit101.components.Panel;
	import com.bit101.components.PushButton;
	import com.hisschemoller.audio.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.sequencer.model.enum.PlaybackStatus;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.enum.UILayout;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;

	/**
	 * @author wouter.hisschemoller
	 * Oct 13, 2010
	 */
	public class Controls extends Panel
	{
		private var _playButton : PushButton;
		private var _volumeKnob : Knob;
		private var _tempoKnob : Knob;
		private var _meterL : Meter;
		private var _meterR : Meter;
		private var _isPlaying : Boolean;

		public function Controls ( parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0 )
		{
			super( parent, xpos, ypos );
		}

		/**
		 * 
		 */
		public function updateMeters (streamInfoVO : AudioStreamInfoVO) : void
		{
			_meterL.value = streamInfoVO.peakLevelLeft;
			_meterR.value = streamInfoVO.peakLevelRight;
		}

		/**
		 * 
		 */
		public function updatePlaybackState (playbackStatus : String) : void
		{
			_isPlaying = (playbackStatus == PlaybackStatus.PLAYING);
			
			if(_isPlaying)
			{
				_playButton.label = "Stop";
			}
			else
			{
				_playButton.label = "Play";
				_meterL.value = 0;
				_meterR.value = 0;
			}
		}

		/**
		 * 
		 */
		public function updateTempo (value : Number) : void
		{
			_tempoKnob.value = value;
		}

		/**
		 * 
		 */
		public function updateVolumeNormalized (volumeNormalized : Number) : void
		{
			_volumeKnob.value = _volumeKnob.minimum + (volumeNormalized * (_volumeKnob.maximum - _volumeKnob.minimum));
		}

		/**
		 * 
		 */
		public function getTempoKnobValue () : Number
		{
			return _tempoKnob.value;
		}

		/**
		 * 
		 */
		public function getVolumeKnobValueNormalized () : Number
		{
			return (_volumeKnob.value - _volumeKnob.minimum) / (_volumeKnob.maximum - _volumeKnob.minimum);
		}

		/**
		 * 
		 */
		override protected function init () : void
		{
			super.init( );
			
			move( UILayout.MARGIN, UILayout.MARGIN );
			setSize( stage.stageWidth - ( UILayout.MARGIN * 2 ), UILayout.CONTROLS_HEIGHT );
		}

		/**
		 * 
		 */
		override protected function addChildren () : void
		{
			super.addChildren( );
			
			_playButton = new PushButton( this, 10, 30, "Play", controlChangeHandler );
			_playButton.setSize( 60, _playButton.height );
			
			_tempoKnob = new Knob( this, 80, 0, "Tempo", controlChangeHandler );
			_tempoKnob.minimum = 50;
			_tempoKnob.maximum = 170;
			
			_volumeKnob = new Knob( this, 130, 0, "Volume", controlChangeHandler );
			_volumeKnob.minimum = 0;
			_volumeKnob.maximum = 10;
			
			_meterL = new Meter( this, 255, 10, "left" );
			_meterL.setSize( 120, NaN );
			
			_meterR = new Meter( this, 380, 10, "right" );
			_meterR.setSize( 120, NaN );
			
			new Label( this, 10, 60, "v1.01" );
		}

		/**
		 * 
		 */
		private function controlChangeHandler (event : Event) : void
		{
			switch(event.target)
			{
				case _playButton:
					var eventType : String = _isPlaying ? UIEventType.PLAYBACK_STOP : UIEventType.PLAYBACK_START;
					dispatchEvent( new Event( eventType ) );
					break;
					
				case _tempoKnob:
					dispatchEvent( new Event( UIEventType.TEMPO_CHANGE ) );
					break;
					
				case _volumeKnob:
					dispatchEvent( new Event( UIEventType.MASTER_VOLUME_CHANGE ) );
					break;
			}
		}
	}
}
