package com.hisschemoller.sequencer.view.components.euclidpattern 
{
	import com.bit101.components.Component;
	import com.greensock.TweenLite;
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternEventViewVO;
	import com.hisschemoller.sequencer.model.vo.PatternViewVO;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.events.EuclidPatternEvent;
	import com.hisschemoller.utils.IDisposable;

	import flash.display.DisplayObjectContainer;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	/**
	 * @author wouterhisschemoller
	 */
	public class EuclidPattern extends Component implements IDisposable
	{
		private static const DOUBLE_PI : Number = Math.PI * 2;
		private var _id : String;
		private var _polygon : Shape;
		private var _pointer : Shape;
		private var _steps : Vector.<EuclidStep> = new Vector.<EuclidStep>( );
		private var _hits : Vector.<PatternEventViewVO> = new Vector.<PatternEventViewVO>( );
		private var _center : Sprite;
		private var _select : Sprite;
		private var _mouseOffsetX : Number;
		private var _mouseOffsetY : Number;
		private var _patternLengthInParts : int;
		private var _millisecondsPerPart : Number;
		private var _timeAtProcessStart : Number;
		private var _patternPositionInParts : Number;

		public function EuclidPattern ( parent : DisplayObjectContainer = null )
		{
			super( parent );
		}

		public function dispose () : void
		{
			_center.removeEventListener( MouseEvent.DOUBLE_CLICK, centerDoubleClickHandler );
			_center.removeEventListener( MouseEvent.MOUSE_DOWN, centerMouseDownHandler );
			stage.removeEventListener( MouseEvent.MOUSE_UP, centerMouseUpHandler );
			stage.removeEventListener( MouseEvent.MOUSE_MOVE, centerMouseMoveHandler );
		}

		/**
		 * Create a new pattern.
		 */
		public function createPattern ( patternVO : EuclidPatternVO ) : void
		{
			_id = patternVO.id;
			_patternLengthInParts = patternVO.length;
			
			move( patternVO.shape.x, patternVO.shape.y );
			
			drawPattern( patternVO );
			
			/** Animation. */
			var n : uint = _steps.length;
			for( var i : uint = 0; i < n; i++)
			{
				TweenLite.from( _steps[ i ], 0.3, { x:0, y:0, alpha:0.3 } );
			}
			
			TweenLite.from( _polygon, 0.3, { scaleX:0.1, scaleY:0.1, alpha:0.3 } );
		}

		/**
		 * Update an existing pattern with changed data.
		 */
		public function updatePattern ( patternVO : EuclidPatternVO ) : void
		{
			_patternLengthInParts = patternVO.length;
			
			/** Remove the old steps. */
			var n : int = _steps.length;
			while( --n > -1 )
			{
				var step : EuclidStep = _steps.pop( );
				step.dispose( );
				removeChild( step );
			}
			
			/** Clear the polygon. */
			_polygon.graphics.clear( );
			
			drawPattern( patternVO );
		}

		/**
		 * From SequencerProxy after a pattern is processed.
		 */
		public function updateViewData ( patternViewVO : PatternViewVO ) : void
		{
			_patternPositionInParts = patternViewVO.positionInParts;
			_timeAtProcessStart = patternViewVO.timeAtProcessStart;
			_hits = patternViewVO.events;
		}

		/**
		 * Update pointer animation (on ENTER_FRAME).
		 */
		public function updateView ( latencyInMs : Number ) : void
		{
			var offsetInMs : Number = new Date( ).getTime( ) - _timeAtProcessStart;
			var offsetTotalInParts : Number = ( offsetInMs - latencyInMs ) / _millisecondsPerPart;
			var positionRelative : Number = ( _patternPositionInParts + offsetTotalInParts ) / _patternLengthInParts;
			
			_pointer.rotation = positionRelative * 360;
			
			if( _hits.length > 0 )
			{
				var n : int = _hits.length;
				while( --n > -1 )
				{
					var patternEventViewVO : PatternEventViewVO = _hits.pop( );
					_steps[ patternEventViewVO.stepIndex ].hit( patternEventViewVO.offsetInMs + offsetInMs + latencyInMs );
				}
			}
		}

		public function select ( isSelected : Boolean ) : void
		{
			if( _select.visible != isSelected)
			{
				if( isSelected )
				{
					TweenLite.to( _select, 0.5, { autoAlpha:1 } );
				}
				else
				{
					TweenLite.to( _select, 0.5, { autoAlpha:0 } );
				}
			}
		}

		/**
		 * Tempo has changed. 
		 * _millisecondsPerPart is used in the time indicator animation.
		 */
		public function updateTempo ( millisecondsPerPart : Number) : void
		{
			_millisecondsPerPart = millisecondsPerPart;
		}

		public function get id () : String
		{
			return _id;
		}

		public function get mouseOffsetX () : Number
		{
			return _mouseOffsetX;
		}

		public function get mouseOffsetY () : Number
		{
			return _mouseOffsetY;
		}

		/**
		 * Overriden in subclasses to create child display objects.
		 */
		override protected function addChildren () : void
		{
			_polygon = new Shape( );
			addChild( _polygon );
			
			_pointer = new Shape( );
			_pointer.graphics.lineStyle( 2, 0x666666 );
			_pointer.graphics.lineTo( 0, -50 );
			// _pointer.graphics.drawEllipse( -4, -50 - 4, 8, 8 );
			addChild( _pointer );
			
			/** Center circle. */
			_center = new Sprite( );
			_center.graphics.lineStyle( 2, 0x666666 );
			_center.graphics.beginFill( 0xFFFFFF );
			_center.graphics.drawEllipse( -20, -20, 40, 40 );
			_center.doubleClickEnabled = true;
			_center.buttonMode = true;
			_center.addEventListener( MouseEvent.DOUBLE_CLICK, centerDoubleClickHandler );
			_center.addEventListener( MouseEvent.MOUSE_DOWN, centerMouseDownHandler );
			addChild( _center );
			
			/** Select indicator. */
			_select = new Sprite( );
			_select.graphics.lineStyle( 2, 0x999999 );
			_select.graphics.drawEllipse( -16, -16, 32, 32 );
			addChild( _select );
			
			TweenLite.from( _pointer, 0.3, { scaleX:0.5, scaleY:0.5 } );
			TweenLite.from( _center, 0.3, { scaleX:0.5, scaleY:0.5 } );
			TweenLite.from( _select, 0.3, { scaleX:0.5, scaleY:0.5 } );
		}

		private function drawPattern ( patternVO : EuclidPatternVO ) : void
		{
			_polygon.graphics.lineStyle( 1.5, 0xCCCCCC );
			_polygon.graphics.beginFill( 0x000000, 0.02 );
			var polygonStarted : Boolean = false;
			
			/** The step circles */
			var interval : Number = DOUBLE_PI / patternVO.steps;
			var n : uint = patternVO.steps;
			for( var i : uint = 0; i < n; i++)
			{
				/** See if the step is selected. */
				var selected : Boolean = false;
				var m : int = patternVO.events.length;
				while( --m > -1 )
				{
					if( patternVO.events[ m ].time / patternVO.quantization == i )
					{
						selected = true;
						break;
					}
				}
				
				/** Create the step. */
				var position : Number = i * interval;
				var step : EuclidStep = new EuclidStep( this, 0, 0, selected );
				step.x = Math.sin( position ) * 50;
				step.y = Math.cos( position ) * -50;
				
				/** Draw the polygon. */
				if( selected )
				{
					if( !polygonStarted )
					{
						polygonStarted = true;
						_polygon.graphics.moveTo( step.x, step.y );
					}
					else
					{
						_polygon.graphics.lineTo( step.x, step.y );
					}
				}
				
				_steps.push( step );
			}
			
			_polygon.graphics.endFill( );
		}

		private function centerMouseDownHandler (event : MouseEvent) : void 
		{
			_mouseOffsetX = mouseX;
			_mouseOffsetY = mouseY;
			stage.addEventListener( MouseEvent.MOUSE_UP, centerMouseUpHandler );
			stage.addEventListener( MouseEvent.MOUSE_MOVE, centerMouseMoveHandler );
		}

		private function centerMouseUpHandler (event : MouseEvent) : void 
		{
			stage.removeEventListener( MouseEvent.MOUSE_UP, centerMouseUpHandler );
			stage.removeEventListener( MouseEvent.MOUSE_MOVE, centerMouseMoveHandler );
		}

		private function centerMouseMoveHandler (event : MouseEvent) : void 
		{
			dispatchEvent( new Event( UIEventType.PATTERN_MOVE ) );
		}

		/**
		 * DOUBLE_CLICK on _center (to open pattern settings window).
		 */
		private function centerDoubleClickHandler (event : MouseEvent) : void 
		{
			dispatchEvent( new EuclidPatternEvent( EuclidPatternEvent.EDIT_PATTERN_SETTINGS, _id, true ) );
		}
	}
}
