package com.hisschemoller.sequencer.view.components.linearpattern 
{
	import com.bit101.components.Panel;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.view.events.SequencerEvent;
	import com.newgonzo.midi.messages.MessageStatus;
	import com.newgonzo.midi.messages.VoiceMessage;

	import flash.display.DisplayObjectContainer;

	/**
	 * @author wouterhisschemoller
	 */
	public class PatternEditorPatternBase extends Panel 
	{
		protected var _steps : Vector.<PatternEditorStep> = new Vector.<PatternEditorStep>();
		protected var _quantization : uint;
		protected var _length : uint;
		protected var _zoomLevel : Number = 1;
		protected var _minimalStepButtonWidth : Number = 10;
		
		public function PatternEditorPatternBase (parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0, h : Number = 0)
		{
			super( parent, xpos, ypos );
			
			_height = h;
		}
		
		/**
		 * Zoom level changed.
		 * @param zoomLevel: 1 means one part is one pixel, 2 is two pixels per part.
		 */
		public function updateZoomLevel(zoomLevel : Number) : void
		{
			_zoomLevel = zoomLevel;
			_width = _length * _zoomLevel;
		}
		
		/**
		 * 
		 */
		public function updatePattern(patternVO : PatternVO) : void
		{
			_quantization = patternVO.quantization;
			_length = patternVO.length;
		}

		/**
		 * 
		 */
		public function addStep(seqEventVO : SeqEventVO) : PatternEditorStep
		{
			var numSteps : uint = _length / _quantization;
			var stepWidth : Number = Math.max(_minimalStepButtonWidth, (_length / numSteps) * _zoomLevel);
			var xPos : Number = (seqEventVO.time / _quantization) * ( _quantization * _zoomLevel );
			var step : PatternEditorStep = new PatternEditorStep(seqEventVO, this, xPos);
			step.setSize( stepWidth, _height );
			_steps.push(step);
			
			return step;
		}

		/**
		 * 
		 */
		public function deleteStep(seqEventVO : SeqEventVO) : void
		{
			var n : uint = _steps.length;
			while(--n > -1)
			{
				var step : PatternEditorStep = _steps[n];
				if(step.seqEventVO == seqEventVO)
				{
					step.dispose();
					_steps.splice(n, 1);
					removeChild(step);
					return;
				}
			}
		}
		
		/**
		 * 
		 */
		protected function createNewEvent(xPosition : Number) : void
		{
			var seqEventVO : SeqEventVO = new SeqEventVO();
			seqEventVO.message = new VoiceMessage(MessageStatus.NOTE_ON, 0);
			seqEventVO.time = getTimeAtXPosition(xPosition);
			dispatchEvent(new SequencerEvent(SequencerEvent.EVENT_CREATE , seqEventVO, true) );
		}
		
		/**
		 * @param xPosition: The x position in pixels to convert.
		 * @return Time (in PPQN) at the x position.
		 */
		protected function getTimeAtXPosition(xPosition : Number) : int
		{
			var time : uint = xPosition / _zoomLevel;
			var timeQuantisized : uint = uint(time / _quantization) * _quantization;
			var timeOffsetCorrected : uint = timeQuantisized % _length;
			
			return timeOffsetCorrected;
		}
	}
}
