package com.hisschemoller.sequencer.view.components.linearpattern 
{
	import com.bit101.components.Component;
	import com.bit101.components.PushButton;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.events.SequencerEvent;
	import com.hisschemoller.utils.IDisposable;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;

	/**
	 * @author wouter.hisschemoller
	 * Oct 19, 2010
	 */
	public class PatternEditorTrack extends Component implements IDisposable
	{
		public static const PATTERN_X : int = 80;
		public static const HEIGHT : int = 19;
		private var _id : String;
		private var _pattern : PatternEditorPatternBase;
		private var _editButton : PushButton;

		public function PatternEditorTrack ( id : String, parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0 ) 
		{
			_id = id;

			super( parent, xpos, ypos );
		}

		/**
		 * Clean up before deletion.
		 */
		public function dispose () : void
		{
			_pattern.removeEventListener( SequencerEvent.EVENT_CREATE, sequencerEventHandler );
			_pattern.removeEventListener( SequencerEvent.EVENT_DELETE, sequencerEventHandler );
		}

		/**
		 * @param patternVO: The pattern to display.
		 * @param zoomLevel: 1 means 1 pixel = 1 tick.
		 */
		public function updatePattern ( patternVO : PatternVO ) : void 
		{
			if(patternVO.quantization && patternVO.quantization >= 3)
			{
				_pattern = new PatternEditorPatternFixed( this, PATTERN_X, 0, HEIGHT );
			}
			else
			{
				_pattern = new PatternEditorPatternFree( this, PATTERN_X, 0, HEIGHT );
			}
			
			_pattern.updatePattern( patternVO );
			_pattern.addEventListener( SequencerEvent.EVENT_CREATE, sequencerEventHandler );
			_pattern.addEventListener( SequencerEvent.EVENT_DELETE, sequencerEventHandler );
		}

		/**
		 * @return ID of this track.
		 */
		public function get id () : String 
		{
			return _id;
		}

		/**
		 * Exposes pattern. Used for convenience by PatternEditor.
		 */
		public function get pattern () : PatternEditorPatternBase
		{
			return _pattern;
		}

		/**
		 * Add displayObjects, part of the startup cyclus.
		 */
		override protected function addChildren () : void 
		{
			super.addChildren( );
			
			_editButton = new PushButton( this, 10, 0, "Edit", editButtonClickHandler );
			_editButton.setSize( 60, _editButton.height );
			_editButton.enabled = true;
			_editButton.visible = true;
		}

		/**
		 * Add displayObjects, part of the startup cyclus.
		 */
		private function editButtonClickHandler ( event : MouseEvent ) : void 
		{
			dispatchEvent( new Event( UIEventType.TRACK_EDIT_CLICK, true ) );
		}

		/**
		 * @param event: SequencerEvents bubble down from the pattern. 
		 * The trach ID is added to the event here, then it bubbles further.
		 */
		private function sequencerEventHandler (event : SequencerEvent) : void 
		{
			event.seqEventVO.trackID = _id;
		}
	}
}
