package com.hisschemoller.sequencer.view.components.settings 
{
	import com.bit101.components.ComboBox;
	import com.bit101.components.HSlider;
	import com.bit101.components.Label;
	import com.bit101.components.Window;
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.TrackVO;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.enum.UILayout;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;

	/**
	 * @author wouterhisschemoller
	 */
	public class EuclidPatternSettings extends Window 
	{
		private var _stepsSlider : HSlider;
		private var _stepsLabel : Label;
		private var _fillsSlider : HSlider;
		private var _fillsLabel : Label;
		private var _trackComboBox : ComboBox;
		private var _patternID : String;

		public function EuclidPatternSettings (parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0)
		{
			super( parent, xpos, ypos );
		}

		public function updatePattern (patternVO : EuclidPatternVO) : void
		{
			_patternID = patternVO.id;
			_stepsSlider.value = patternVO.length / patternVO.quantization;
			_stepsLabel.text = String( patternVO.length / patternVO.quantization );
			_fillsSlider.value = patternVO.events.length;
			_fillsLabel.text = String( patternVO.events.length );
			
			var listItems : Array = _trackComboBox.items;
			var n : int = listItems.length;
			while( --n > -1 )
			{
				var item : ComboBoxItem = listItems[ n ] as ComboBoxItem;
				if( item.trackID == patternVO.trackID )
				{
					_trackComboBox.selectedItem = item;
				}
			}
		}

		/**
		 * 
		 */
		public function addTrackToComboBox ( trackVO : TrackVO ) : void
		{
			_trackComboBox.addItem( new ComboBoxItem( trackVO.id, trackVO.label ) );
		}

		/**
		 * 
		 */
		public function getSettings () : EuclidPatternVO
		{
			var patternVO : EuclidPatternVO = new EuclidPatternVO( );
			patternVO.id = _patternID;
			patternVO.steps = _stepsSlider.value;
			patternVO.fills = _fillsSlider.value;
			
			return patternVO;
		}

		/**
		 * Return PatternVO with the changed trackID.
		 */
		public function getTrackSettings () : PatternVO
		{
			var patternVO : PatternVO = new PatternVO( );
			patternVO.id = _patternID;
			patternVO.trackID = ( _trackComboBox.selectedItem as ComboBoxItem ).trackID;
			
			return patternVO;
		}

		public function get numSteps () : int
		{
			return _stepsSlider.value;
		}

		public function get numFills () : int
		{
			return _fillsSlider.value;
		}

		/**
		 * 
		 */
		override public function draw () : void
		{
			super.draw( );
			
			_stepsSlider.setSize( UILayout.SETTINGS_WIDTH - ( UILayout.SETTINGS_H_PADDING * 2 ) - 20 - 30, _stepsSlider.height );
			_stepsLabel.x = UILayout.SETTINGS_WIDTH - 30;
			_fillsSlider.setSize( UILayout.SETTINGS_WIDTH - ( UILayout.SETTINGS_H_PADDING * 2 ) - 20 - 30, _fillsSlider.height );
			_fillsLabel.x = UILayout.SETTINGS_WIDTH - 30;
		}

		/**
		 * Initilizes the component.
		 */
		override protected function init () : void
		{
			super.init( );
			
			title = "Pattern Settings";
			shadow = false;
			hasMinimizeButton = true;
			_minimizeButton.removeEventListener( MouseEvent.CLICK, onMinimize );
		}

		/**
		 * Create child display objects.
		 */
		override protected function addChildren () : void
		{
			super.addChildren( );
			
			var label : Label = new Label( _panel, UILayout.SETTINGS_H_PADDING, 20, "Steps" );
			
			_stepsSlider = new HSlider( _panel, UILayout.SETTINGS_COMPONENT_X, 20 + 5, stepsSliderChange );
			_stepsSlider.setSize( UILayout.SETTINGS_COMPONENT_W, _stepsSlider.height );
			_stepsSlider.minimum = 1;
			_stepsSlider.maximum = 16;
			_stepsSlider.tick = 1;
			
			_stepsLabel = new Label( _panel, UILayout.SETTINGS_WIDTH - 30, 20, "0" );
			
			label = new Label( _panel, UILayout.SETTINGS_H_PADDING, 40, "Fills" );
			
			_fillsSlider = new HSlider( _panel, UILayout.SETTINGS_COMPONENT_X, 40 + 5, fillsSliderChange );
			_fillsSlider.setSize( UILayout.SETTINGS_COMPONENT_W, _fillsSlider.height );
			_fillsSlider.minimum = 1;
			_fillsSlider.maximum = 16;
			_fillsSlider.tick = 1;
			
			_fillsLabel = new Label( _panel, UILayout.SETTINGS_WIDTH - 30, 40, "0" );
			
			label = new Label( _panel, UILayout.SETTINGS_H_PADDING, 65, "Sound" );
			
			_trackComboBox = new ComboBox( _panel, UILayout.SETTINGS_COMPONENT_X, 65, "Select sound" );
			_trackComboBox.setSize( UILayout.SETTINGS_COMPONENT_W, _trackComboBox.height );
			_trackComboBox.addEventListener( Event.SELECT, trackSelectHandler );
			
			label = new Label( _panel, 20, 100, "Use the sliders to change the pattern \nlength and amount of hits.\n\nUse the dropdown menu to change \nthe sound this pattern plays." );
			label.textField.multiline = true;
			label.textField.wordWrap = true;
			label.width = UILayout.SETTINGS_WIDTH - (UILayout.SETTINGS_H_PADDING * 2);
			label.height = 100;
		}

		/**
		 * Steps slider was changed.
		 */
		private function stepsSliderChange ( event : Event ) : void 
		{
			var numFills : int = _fillsSlider.value;
			
			_stepsLabel.text = String( _stepsSlider.value );
			_fillsSlider.maximum = _stepsSlider.value;
			
			if( _fillsSlider.value != numFills )
			{
				fillsSliderChange( new Event( Event.CHANGE ) );
			}
			
			dispatchEvent( new Event( UIEventType.PATTERN_CHANGE ) );
		}

		/**
		 * Fills slider was changed.
		 */
		private function fillsSliderChange ( event : Event ) : void 
		{
			_fillsLabel.text = String( _fillsSlider.value );
			
			dispatchEvent( new Event( UIEventType.PATTERN_CHANGE ) );
		}
		
		/**
		 * An item in the track comboBox was selected.
		 */
		private function trackSelectHandler (event : Event) : void 
		{
			dispatchEvent( new Event( UIEventType.TRACK_CHANGE ) );
		}
	}
}

internal class ComboBoxItem
{
	public var trackID : String;
	public var label : String;

	public function ComboBoxItem ( trackID : String, label : String )
	{
		this.trackID = trackID;
		this.label = label;
	}
}
